@@ -50,6 +50,8 @@ struct _GisAccountPageLocalPrivate
   GtkWidget *header;
   GtkWidget *fullname_entry;
   GtkWidget *username_combo;
+  GtkWidget *enable_parental_controls_box;
+  GtkWidget *enable_parental_controls_check_button;
   gboolean   has_custom_username;
   GtkWidget *username_explanation;
   UmPhotoDialog *photo_dialog;
@@ -59,7 +61,6 @@ struct _GisAccountPageLocalPrivate
   GdkPixbuf *avatar_pixbuf;
   gchar *avatar_filename;
 
-  ActUser *act_user;
   ActUserManager *act_client;
 
   GoaClient *goa_client;
@@ -74,7 +75,8 @@ G_DEFINE_TYPE_WITH_PRIVATE (GisAccountPageLocal, gis_account_page_local, GTK_TYP
 
 enum {
   VALIDATION_CHANGED,
-  USER_CREATED,
+  MAIN_USER_CREATED,
+  PARENT_USER_CREATED,
   CONFIRM,
   LAST_SIGNAL,
 };
@@ -251,6 +253,7 @@ validate (GisAccountPageLocal *page)
   GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
   GtkWidget *entry;
   const gchar *name, *username;
+  gboolean parental_controls_enabled;
   gchar *tip;
 
   if (priv->timeout_id != 0) {
@@ -262,12 +265,17 @@ validate (GisAccountPageLocal *page)
 
   name = gtk_entry_get_text (GTK_ENTRY (priv->fullname_entry));
   username = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (priv->username_combo));
+#ifdef HAVE_PARENTAL_CONTROLS
+  parental_controls_enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->enable_parental_controls_check_button));
+#else
+  parental_controls_enabled = FALSE;
+#endif
 
   priv->valid_name = is_valid_name (name);
   if (priv->valid_name)
     set_entry_validation_checkmark (GTK_ENTRY (priv->fullname_entry));
 
-  priv->valid_username = is_valid_username (username, &tip);
+  priv->valid_username = is_valid_username (username, parental_controls_enabled, &tip);
   if (priv->valid_username)
     set_entry_validation_checkmark (GTK_ENTRY (entry));
 
@@ -355,28 +363,29 @@ avatar_callback (GdkPixbuf   *pixbuf,
 {
   GisAccountPageLocal *page = user_data;
   GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
-  GdkPixbuf *tmp, *rounded;
+  g_autoptr(GdkPixbuf) tmp = NULL;
+  g_autoptr(GdkPixbuf) rounded = NULL;
 
   g_clear_object (&priv->avatar_pixbuf);
-  g_free (priv->avatar_filename);
-  priv->avatar_filename = NULL;
+  g_clear_pointer (&priv->avatar_filename, g_free);
 
   if (pixbuf) {
     priv->avatar_pixbuf = g_object_ref (pixbuf);
-    tmp = round_image (pixbuf);
-    gtk_image_set_from_pixbuf (GTK_IMAGE (priv->avatar_image), tmp);
-    g_object_unref (tmp);
+    rounded = round_image (pixbuf);
   }
   else if (filename) {
     priv->avatar_filename = g_strdup (filename);
     tmp = gdk_pixbuf_new_from_file_at_size (filename, 96, 96, NULL);
-    rounded = round_image (tmp);
-    g_object_unref (tmp);
 
+    if (tmp != NULL)
+      rounded = round_image (tmp);
+  }
+
+  if (rounded != NULL) {
     gtk_image_set_from_pixbuf (GTK_IMAGE (priv->avatar_image), rounded);
-    g_object_unref (rounded);
   }
   else {
+    /* Fallback. */
     gtk_image_set_pixel_size (GTK_IMAGE (priv->avatar_image), 96);
     gtk_image_set_from_icon_name (GTK_IMAGE (priv->avatar_image), "avatar-default-symbolic", 1);
   }
@@ -389,6 +398,22 @@ confirm (GisAccountPageLocal *page)
     g_signal_emit (page, signals[CONFIRM], 0);
 }
 
+static void
+enable_parental_controls_check_button_toggled_cb (GtkToggleButton *toggle_button,
+                                                  gpointer         user_data)
+{
+  GisAccountPageLocal *page = GIS_ACCOUNT_PAGE_LOCAL (user_data);
+  GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
+  gboolean parental_controls_enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->enable_parental_controls_check_button));
+
+  /* This sets the account type of the main user. When we save_data(), we create
+   * two users if parental controls are enabled: the first user is always an
+   * admin, and the second user is the main user using this @account_type. */
+  priv->account_type = parental_controls_enabled ? ACT_USER_ACCOUNT_TYPE_STANDARD : ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR;
+
+  validate (page);
+}
+
 static void
 gis_account_page_local_constructed (GObject *object)
 {
@@ -414,6 +439,13 @@ gis_account_page_local_constructed (GObject *object)
                             "activate", G_CALLBACK (confirm), page);
   g_signal_connect_swapped (priv->fullname_entry, "activate",
                             G_CALLBACK (confirm), page);
+  g_signal_connect (priv->enable_parental_controls_check_button, "toggled",
+                    G_CALLBACK (enable_parental_controls_check_button_toggled_cb), page);
+
+  /* Disable parental controls if support is not compiled in. */
+#ifndef HAVE_PARENTAL_CONTROLS
+  gtk_widget_hide (priv->enable_parental_controls_box);
+#endif
 
   priv->valid_name = FALSE;
   priv->valid_username = FALSE;
@@ -472,7 +504,8 @@ gis_account_page_local_dispose (GObject *object)
 }
 
 static void
-set_user_avatar (GisAccountPageLocal *page)
+set_user_avatar (GisAccountPageLocal *page,
+                 ActUser             *user)
 {
   GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
   GFile *file = NULL;
@@ -481,7 +514,7 @@ set_user_avatar (GisAccountPageLocal *page)
   GError *error = NULL;
 
   if (priv->avatar_filename != NULL) {
-    act_user_set_icon_file (priv->act_user, priv->avatar_filename);
+    act_user_set_icon_file (user, priv->avatar_filename);
     return;
   }
 
@@ -497,7 +530,7 @@ set_user_avatar (GisAccountPageLocal *page)
   if (!gdk_pixbuf_save_to_stream (priv->avatar_pixbuf, stream, "png", NULL, &error, NULL))
     goto out;
 
-  act_user_set_icon_file (priv->act_user, g_file_get_path (file));
+  act_user_set_icon_file (user, g_file_get_path (file));
 
  out:
   if (error != NULL) {
@@ -509,29 +542,46 @@ set_user_avatar (GisAccountPageLocal *page)
 }
 
 static void
-local_create_user (GisAccountPageLocal *page)
+local_create_user (GisAccountPageLocal *local,
+                   GisPage             *page)
 {
-  GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
+  GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (local);
   const gchar *username;
   const gchar *fullname;
-  GError *error = NULL;
+  g_autoptr(GError) local_error = NULL;
+  gboolean parental_controls_enabled;
+  g_autoptr(ActUser) main_user = NULL;
+  g_autoptr(ActUser) parent_user = NULL;
 
   username = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (priv->username_combo));
   fullname = gtk_entry_get_text (GTK_ENTRY (priv->fullname_entry));
+  parental_controls_enabled = gis_driver_get_parental_controls_enabled (page->driver);
 
-  priv->act_user = act_user_manager_create_user (priv->act_client, username, fullname, priv->account_type, &error);
-  if (error != NULL) {
-    g_warning ("Failed to create user: %s", error->message);
-    g_error_free (error);
+  /* Always create the admin user first, in case of failure part-way through
+   * this function, which would leave us with no admin user at all. */
+  if (parental_controls_enabled) {
+    const gchar *parent_username = "administrator";
+    const gchar *parent_fullname = _("Administrator");
+
+    parent_user = act_user_manager_create_user (priv->act_client, parent_username, parent_fullname, ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR, &local_error);
+    if (local_error != NULL) {
+      g_warning ("Failed to create parent user: %s", local_error->message);
+      return;
+    }
+
+    g_signal_emit (local, signals[PARENT_USER_CREATED], 0, parent_user, "");
+  }
+
+  /* Now create the main user. */
+  main_user = act_user_manager_create_user (priv->act_client, username, fullname, priv->account_type, &local_error);
+  if (local_error != NULL) {
+    g_warning ("Failed to create user: %s", local_error->message);
     return;
   }
 
-  act_user_set_user_name (priv->act_user, username);
-  act_user_set_account_type (priv->act_user, priv->account_type);
+  set_user_avatar (local, main_user);
 
-  set_user_avatar (page);
-
-  g_signal_emit (page, signals[USER_CREATED], 0, priv->act_user, "");
+  g_signal_emit (local, signals[MAIN_USER_CREATED], 0, main_user, "");
 }
 
 static void
@@ -547,6 +597,8 @@ gis_account_page_local_class_init (GisAccountPageLocalClass *klass)
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, fullname_entry);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, username_combo);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, username_explanation);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, enable_parental_controls_box);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAccountPageLocal, enable_parental_controls_check_button);
 
   object_class->constructed = gis_account_page_local_constructed;
   object_class->dispose = gis_account_page_local_dispose;
@@ -555,9 +607,13 @@ gis_account_page_local_class_init (GisAccountPageLocalClass *klass)
                                               G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
                                               G_TYPE_NONE, 0);
 
-  signals[USER_CREATED] = g_signal_new ("user-created", GIS_TYPE_ACCOUNT_PAGE_LOCAL,
-                                        G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
-                                        G_TYPE_NONE, 2, ACT_TYPE_USER, G_TYPE_STRING);
+  signals[MAIN_USER_CREATED] = g_signal_new ("main-user-created", GIS_TYPE_ACCOUNT_PAGE_LOCAL,
+                                             G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
+                                             G_TYPE_NONE, 2, ACT_TYPE_USER, G_TYPE_STRING);
+
+  signals[PARENT_USER_CREATED] = g_signal_new ("parent-user-created", GIS_TYPE_ACCOUNT_PAGE_LOCAL,
+                                               G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
+                                               G_TYPE_NONE, 2, ACT_TYPE_USER, G_TYPE_STRING);
 
   signals[CONFIRM] = g_signal_new ("confirm", GIS_TYPE_ACCOUNT_PAGE_LOCAL,
                                    G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
@@ -580,20 +636,44 @@ gis_account_page_local_validate (GisAccountPageLocal *page)
 }
 
 void
-gis_account_page_local_create_user (GisAccountPageLocal *page)
+gis_account_page_local_create_user (GisAccountPageLocal *local,
+                                    GisPage             *page)
 {
-  local_create_user (page);
+  local_create_user (local, page);
 }
 
 gboolean
 gis_account_page_local_apply (GisAccountPageLocal *local, GisPage *page)
 {
   GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (local);
-  const gchar *username;
+  const gchar *username, *full_name;
+  gboolean parental_controls_enabled;
 
   username = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (priv->username_combo));
   gis_driver_set_username (GIS_PAGE (page)->driver, username);
 
+  full_name = gtk_entry_get_text (GTK_ENTRY (priv->fullname_entry));
+  gis_driver_set_full_name (GIS_PAGE (page)->driver, full_name);
+
+  if (priv->avatar_pixbuf != NULL)
+    {
+      gis_driver_set_avatar (GIS_PAGE (page)->driver, priv->avatar_pixbuf);
+    }
+  else if (priv->avatar_filename != NULL)
+    {
+      g_autoptr(GdkPixbuf) pixbuf = NULL;
+
+      pixbuf = gdk_pixbuf_new_from_file_at_size (priv->avatar_filename, 96, 96, NULL);
+      gis_driver_set_avatar (GIS_PAGE (page)->driver, pixbuf);
+    }
+
+#ifdef HAVE_PARENTAL_CONTROLS
+  parental_controls_enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->enable_parental_controls_check_button));
+#else
+  parental_controls_enabled = FALSE;
+#endif
+  gis_driver_set_parental_controls_enabled (GIS_PAGE (page)->driver, parental_controls_enabled);
+
   return FALSE;
 }
 
