@@ -46,15 +46,95 @@ struct _GisPasswordPagePrivate
   GtkWidget *password_strength;
   GtkWidget *password_explanation;
   GtkWidget *confirm_explanation;
+  GtkWidget *header;
+
   gboolean valid_confirm;
   gboolean valid_password;
   guint timeout_id;
   const gchar *username;
+  gboolean parent_mode;
 };
 typedef struct _GisPasswordPagePrivate GisPasswordPagePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GisPasswordPage, gis_password_page, GIS_TYPE_PAGE);
 
+typedef enum
+{
+  PROP_PARENT_MODE = 1,
+} GisPasswordPageProperty;
+
+static GParamSpec *obj_props[PROP_PARENT_MODE + 1];
+
+static void
+update_header (GisPasswordPage *page)
+{
+  GisPasswordPagePrivate *priv = gis_password_page_get_instance_private (page);
+  g_autofree gchar *title = NULL;
+  g_autofree gchar *subtitle = NULL;
+  const gchar *icon_name;
+  GdkPixbuf *pixbuf;
+
+#ifndef HAVE_PARENTAL_CONTROLS
+  /* Don’t break UI compatibility if parental controls are disabled. */
+  title = g_strdup (_("Set a Password"));
+  subtitle = g_strdup (_("Be careful not to lose your password."));
+  pixbuf = NULL;
+  icon_name = "dialog-password-symbolic";
+#else
+  if (!priv->parent_mode)
+    {
+      /* Translators: The placeholder is for the user’s full name. */
+      title = g_strdup_printf (_("Set a Password for %s"),
+                               gis_driver_get_full_name (GIS_PAGE (page)->driver));
+      subtitle = g_strdup (_("Be careful not to lose your password."));
+      pixbuf = gis_driver_get_avatar (GIS_PAGE (page)->driver);
+      icon_name = (pixbuf != NULL) ? NULL : "dialog-password-symbolic";
+    }
+  else
+    {
+      title = g_strdup (_("Set a Parent Password"));
+      /* Translators: The placeholder is the full name of the child user on the system. */
+      subtitle = g_strdup_printf (_("This password will control access to the parental controls for %s."),
+                                  gis_driver_get_full_name (GIS_PAGE (page)->driver));
+      /* FIXME: Use the malcontent icon once it’s available. See
+       *  * https://gitlab.freedesktop.org/pwithnall/malcontent/issues/9
+       *  * https://gitlab.gnome.org/GNOME/gnome-initial-setup/merge_requests/72#note_704634
+       */
+      icon_name = "dialog-password-symbolic";
+      pixbuf = NULL;
+    }
+#endif
+
+  /* Doesn’t make sense to set both. */
+  g_assert (icon_name == NULL || pixbuf == NULL);
+
+  g_object_set (G_OBJECT (priv->header),
+                "title", title,
+                "subtitle", subtitle,
+                NULL);
+  if (pixbuf != NULL)
+    g_object_set (G_OBJECT (priv->header), "pixbuf", pixbuf, NULL);
+  else if (icon_name != NULL)
+    g_object_set (G_OBJECT (priv->header), "icon-name", icon_name, NULL);
+}
+
+static void
+set_parent_mode (GisPasswordPage *page,
+                 gboolean         parent_mode)
+{
+  GisPasswordPagePrivate *priv = gis_password_page_get_instance_private (page);
+
+  g_return_if_fail (GIS_IS_PASSWORD_PAGE (page));
+
+  if (priv->parent_mode == parent_mode)
+    return;
+
+  priv->parent_mode = parent_mode;
+  g_object_notify_by_pspec (G_OBJECT (page), obj_props[PROP_PARENT_MODE]);
+
+  update_header (page);
+}
+
 static gboolean
 page_validate (GisPasswordPage *page)
 {
@@ -83,9 +163,14 @@ gis_password_page_save_data (GisPage *gis_page)
 
   account_mode = gis_driver_get_account_mode (gis_page->driver);
 
-  gis_driver_get_user_permissions (gis_page->driver, &act_user, &password);
+  if (!priv->parent_mode)
+    gis_driver_get_user_permissions (gis_page->driver, &act_user, &password);
+  else
+    gis_driver_get_parent_permissions (gis_page->driver, &act_user, &password);
 
   if (account_mode == UM_ENTERPRISE) {
+    g_assert (!priv->parent_mode);
+
     if (password != NULL)
       gis_update_login_keyring_password (password);
     return;
@@ -98,9 +183,13 @@ gis_password_page_save_data (GisPage *gis_page)
   else
     act_user_set_password (act_user, password, "");
 
-  gis_driver_set_user_permissions (gis_page->driver, act_user, password);
+  if (!priv->parent_mode)
+    gis_driver_set_user_permissions (gis_page->driver, act_user, password);
+  else
+    gis_driver_set_parent_permissions (gis_page->driver, act_user, password);
 
-  gis_update_login_keyring_password (password);
+  if (!priv->parent_mode)
+    gis_update_login_keyring_password (password);
 }
 
 static void
@@ -154,6 +243,13 @@ validate (GisPasswordPage *page)
     }
   }
 
+  /*
+   * We deliberately don’t validate that the parent password and main user
+   * password are different. It’s more feasible that someone would usefully
+   * want to set their system up that way, than it is that the parent and child
+   * would accidentally choose the same password.
+   */
+
   update_page_validation (page);
 
   return FALSE;
@@ -219,6 +315,16 @@ username_changed (GObject *obj, GParamSpec *pspec, GisPasswordPage *page)
   validate (page);
 }
 
+static void
+full_name_or_avatar_changed (GObject    *obj,
+                             GParamSpec *pspec,
+                             gpointer    user_data)
+{
+  GisPasswordPage *page = GIS_PASSWORD_PAGE (user_data);
+
+  update_header (page);
+}
+
 static void
 confirm (GisPasswordPage *page)
 {
@@ -250,18 +356,65 @@ gis_password_page_constructed (GObject *object)
 
   g_signal_connect (GIS_PAGE (page)->driver, "notify::username",
                     G_CALLBACK (username_changed), page);
+  g_signal_connect (GIS_PAGE (page)->driver, "notify::full-name",
+                    G_CALLBACK (full_name_or_avatar_changed), page);
+  g_signal_connect (GIS_PAGE (page)->driver, "notify::avatar",
+                    G_CALLBACK (full_name_or_avatar_changed), page);
 
   validate (page);
+  update_header (page);
 
   gtk_widget_show (GTK_WIDGET (page));
 }
 
+static void
+gis_password_page_get_property (GObject    *object,
+                                guint       prop_id,
+                                GValue     *value,
+                                GParamSpec *pspec)
+{
+  GisPasswordPage *page = GIS_PASSWORD_PAGE (object);
+  GisPasswordPagePrivate *priv = gis_password_page_get_instance_private (page);
+
+  switch ((GisPasswordPageProperty) prop_id)
+    {
+    case PROP_PARENT_MODE:
+      g_value_set_boolean (value, priv->parent_mode);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+gis_password_page_set_property (GObject      *object,
+                                guint         prop_id,
+                                const GValue *value,
+                                GParamSpec   *pspec)
+{
+  GisPasswordPage *page = GIS_PASSWORD_PAGE (object);
+
+  switch ((GisPasswordPageProperty) prop_id)
+    {
+    case PROP_PARENT_MODE:
+      set_parent_mode (page, g_value_get_boolean (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
 static void
 gis_password_page_dispose (GObject *object)
 {
-  if (GIS_PAGE (object)->driver)
-  g_signal_handlers_disconnect_by_func (GIS_PAGE (object)->driver,
-                                        username_changed, object);
+  if (GIS_PAGE (object)->driver) {
+    g_signal_handlers_disconnect_by_func (GIS_PAGE (object)->driver,
+                                          username_changed, object);
+    g_signal_handlers_disconnect_by_func (GIS_PAGE (object)->driver,
+                                          full_name_or_avatar_changed, object);
+  }
 
   G_OBJECT_CLASS (gis_password_page_parent_class)->dispose (object);
 }
@@ -285,6 +438,7 @@ gis_password_page_class_init (GisPasswordPageClass *klass)
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisPasswordPage, password_strength);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisPasswordPage, password_explanation);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisPasswordPage, confirm_explanation);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisPasswordPage, header);
 
   page_class->page_id = PAGE_ID;
   page_class->locale_changed = gis_password_page_locale_changed;
@@ -292,7 +446,27 @@ gis_password_page_class_init (GisPasswordPageClass *klass)
   page_class->shown = gis_password_page_shown;
 
   object_class->constructed = gis_password_page_constructed;
+  object_class->get_property = gis_password_page_get_property;
+  object_class->set_property = gis_password_page_set_property;
   object_class->dispose = gis_password_page_dispose;
+
+  /**
+   * GisPasswordPage:parent-mode:
+   *
+   * If %FALSE (the default), this page will collect a password for the main
+   * user account. If %TRUE, it will collect a password for controlling access
+   * to parental controls — this will affect where the password is stored, and
+   * the appearance of the page.
+   *
+   * Since: 3.36
+   */
+  obj_props[PROP_PARENT_MODE] =
+    g_param_spec_boolean ("parent-mode", "Parent Mode",
+                          "Whether to collect a password for the main user account or a parent account.",
+                          FALSE,
+                          G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+  g_object_class_install_properties (object_class, G_N_ELEMENTS (obj_props), obj_props);
 }
 
 static void
@@ -321,3 +495,15 @@ gis_prepare_password_page (GisDriver *driver)
                        NULL);
 }
 
+GisPage *
+gis_prepare_parent_password_page (GisDriver *driver)
+{
+  /* Skip prompting for the parent password if parental controls aren’t enabled. */
+  if (!gis_driver_get_parental_controls_enabled (driver))
+    return NULL;
+
+  return g_object_new (GIS_TYPE_PASSWORD_PAGE,
+                       "driver", driver,
+                       "parent-mode", TRUE,
+                       NULL);
+}
